@@ -62,6 +62,7 @@ class GenomicsDBConfigBase {
   GenomicsDBConfigBase();
   const std::string& get_workspace(const int rank) const;
   void set_workspace(const std::string& workspace);
+  const bool has_array_name(const int rank) const;
   const std::string& get_array_name(const int rank) const;
   void set_array_name(const std::string& array_name);
   ColumnRange get_column_partition(const int rank, const unsigned idx=0u) const;
@@ -76,6 +77,9 @@ class GenomicsDBConfigBase {
   inline const std::string& get_query_filter() const {
     return m_query_filter;
   }
+  void set_query_filter(const std::string& filter) {
+    m_query_filter = filter;
+  }
   inline size_t get_segment_size() const {
     return m_segment_size;
   }
@@ -170,6 +174,9 @@ class GenomicsDBConfigBase {
   inline bool enable_shared_posixfs_optimizations() const {
     return is_set_with_env_override(m_enable_shared_posixfs_optimizations, ENABLE_SHARED_POSIXFS_OPTIMIZATIONS);
   }
+  inline bool has_annotation_sources() const {
+    return m_has_annotation_sources;
+  }
   void scan_whole_array();
   const std::vector<std::string>& get_attributes() const { return m_attributes; }
   //JSON parsing functions
@@ -182,6 +189,8 @@ class GenomicsDBConfigBase {
   void read_from_PB_binary_string(const std::string& str, const int rank=0);
   int read_from_PB_file(const std::string& filename, const int rank=0) { return -1; }
   void read_from_PB(const genomicsdb_pb::ExportConfiguration* x, const int rank=0);
+  template<typename T>
+  void subset_query_from_PB(const T* x, const int rank=0);
   static int get_pb_from_json_file(google::protobuf::Message* pb_config, const std::string& json_file);
   static bool output_to_stdout(const std::string& name) {
     return name.empty() || name == "-";
@@ -241,6 +250,8 @@ class GenomicsDBConfigBase {
   std::string m_callset_mapping_file;
   //Enable optimizations (disable file locking and enable keep file handles open until finalization
   bool m_enable_shared_posixfs_optimizations;
+  //Annotation Sources specified in protobuf export configuration
+  bool m_has_annotation_sources;
   template<typename T>
   void base_read_from_PB(const T* pb_config);
   template<typename T>
