@@ -6,7 +6,7 @@
  * The MIT License (MIT)
  *
  * Copyright (c) 2019-2020,2022 Omics Data Automation, Inc.
- * Copyright (c) 2023 dātma, inc™
+ * Copyright (c) 2023-2024 dātma, inc™
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
  * this software and associated documentation files (the "Software"), to deal in
@@ -36,6 +36,10 @@
 #include <iostream>
 #include <string>
 
+#ifdef __linux__
+#  include <malloc.h>
+#endif
+
 #include "annotation_service.h"
 #include "broad_combined_gvcf.h"
 #include "query_variants.h"
@@ -166,8 +170,6 @@ GenomicsDB::GenomicsDB(const std::string& query_configuration,
 }
 
 GenomicsDB::~GenomicsDB() {
-  // TODO: delete variant_query_config per array
-
   if (m_annotation_service != nullptr) {
   	delete TO_ANNOTATION_SERVICE(m_annotation_service);
   }
@@ -180,6 +182,9 @@ GenomicsDB::~GenomicsDB() {
   if (m_storage_manager != nullptr) {
     delete TO_VARIANT_STORAGE_MANAGER(m_storage_manager);
   }
+#ifdef __linux__
+  malloc_trim(0);
+#endif
 }
 
 std::map<std::string, genomic_field_type_t> create_genomic_field_types(const VariantQueryConfig &query_config,
@@ -425,6 +430,7 @@ class GatherVariantCalls : public SingleCellOperatorBase {
   void operate(VariantCall& call, const VariantQueryConfig& query_config, const VariantArraySchema& schema);
   void operate_on_columnar_cell(const GenomicsDBColumnarCell& cell, const VariantQueryConfig& query_config,
                                 const VariantArraySchema& schema);
+  void finalize();
  private:
   void initialize(const VariantQueryConfig& query_config);
   GenomicsDBVariantCallProcessor& m_variant_call_processor;
@@ -510,6 +516,10 @@ void GatherVariantCalls::operate_on_columnar_cell(const GenomicsDBColumnarCell&
   m_variant_call_processor.process(sample_name, coords, genomic_interval, genomic_fields);
 }
 
+void GatherVariantCalls::finalize() {
+  m_variant_call_processor.finalize();
+}
+
 #if(DEBUG)
 void print_variant_calls(const VariantQueryConfig& query_config,
                          const VariantQueryProcessor& query_processor,
@@ -542,7 +552,8 @@ std::vector<VariantCall>* GenomicsDB::query_variant_calls(const std::string& arr
 #endif
 
   // Perform Query over all the intervals
-  std::vector<VariantCall> *pvariant_calls = new std::vector<VariantCall>;
+  // This route is not being exercised - std::vector<VariantCall> *pvariant_calls = new std::vector<VariantCall>;
+  std::vector<VariantCall> *pvariant_calls = nullptr;
 
   GatherVariantCalls gather_variant_calls(processor, *query_config, m_annotation_service);
   query_processor->iterate_over_cells(query_processor->get_array_descriptor(), *query_config, gather_variant_calls, true);
@@ -766,7 +777,7 @@ void GenomicsDBResults<genomicsdb_variant_t>::free() {
 
 template<>
 std::size_t GenomicsDBResults<genomicsdb_variant_call_t>::size() const noexcept {
-  return TO_VARIANT_CALL_VECTOR(m_results)->size();
+  return m_results != nullptr?TO_VARIANT_CALL_VECTOR(m_results)->size():0ull;
 }
 
 template<>
