@@ -3,7 +3,7 @@
  *
  * The MIT License (MIT)
  * Copyright (c) 2019-2023 Omics Data Automation, Inc.
- * Copyright (c) 2023 dātma, inc™
+ * Copyright (c) 2023-2024 dātma, inc™
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy of
  * this software and associated documentation files (the "Software"), to deal in
@@ -37,10 +37,15 @@
 #include "test_base.h"
 
 #include <iostream>
+#include <future>
 #include <string>
 #include <thread>
 #include <utility>
 
+#ifdef USE_NANOARROW
+#  include <nanoarrow/nanoarrow.h>
+#endif
+
 TEST_CASE_METHOD(TempDir, "utils", "[genomicsdb_utils]") {
   REQUIRE(genomicsdb::version().size() > 0);
   REQUIRE_THAT(genomicsdb::version(), Catch::Equals(GENOMICSDB_VERSION));
@@ -415,11 +420,6 @@ class TwoQueryIntervalsProcessor : public GenomicsDBVariantCallProcessor {
  public:
   TwoQueryIntervalsProcessor() {};
 
-  ~TwoQueryIntervalsProcessor() {
-    CHECK(m_num_query_intervals == 2);
-    CHECK(m_processed_rows == 5);
-  }
-
   void reinitialize() {
     m_num_query_intervals = 0;
     m_processed_rows = 0;
@@ -443,6 +443,11 @@ class TwoQueryIntervalsProcessor : public GenomicsDBVariantCallProcessor {
     m_processed_rows++;
   };
 
+  void finalize() {
+    CHECK(m_num_query_intervals == 2);
+    CHECK(m_processed_rows == 5);
+  }
+
   int m_num_query_intervals = 0;
   int m_processed_rows = 0;
 };
@@ -1016,6 +1021,159 @@ TEST_CASE("api query_variant_calls with JSONVariantCallProcessor", "[query_varia
   delete gdb;
 }
 
+#ifdef USE_NANOARROW
+
+std::vector<std::string> get_array_value(ArrowArray *array, ArrowType type, int index) {
+  std::vector<std::string> values;
+  ArrowArrayView array_view;
+  ArrowArrayViewInitFromType(&array_view, type);
+  ArrowArrayViewSetArray(&array_view, array->children[index], nullptr);
+  for (auto i=0l; i<array->children[index]->length; i++) {
+    if (ArrowArrayViewIsNull(&array_view, i)) {
+      values.push_back("NULL");
+      continue;
+    } 
+    switch (type) {
+      case NANOARROW_TYPE_STRING: {
+        ArrowStringView item;
+        item = ArrowArrayViewGetStringUnsafe(&array_view, i);
+        values.push_back(std::string(item.data, item.size_bytes));
+        break;
+      }
+      case NANOARROW_TYPE_UINT64: {
+        values.push_back(std::to_string(ArrowArrayViewGetUIntUnsafe(&array_view, i)));
+        break;
+      }
+      case NANOARROW_TYPE_INT32: {
+        
+        values.push_back(std::to_string(ArrowArrayViewGetIntUnsafe(&array_view, i)));
+        break;
+      }
+      case NANOARROW_TYPE_FLOAT: {
+        values.push_back(std::to_string(ArrowArrayViewGetDoubleUnsafe(&array_view, i)));
+        break;
+      }
+      default:
+        CHECK(false);
+    }
+  }
+  return values;
+}
+
+TEST_CASE("api query_variant_calls with ArrowVariantCallProcessor", "[query_variant_calls_with_arrow_output]") {
+  using namespace genomicsdb_pb;
+
+  ExportConfiguration *config = new ExportConfiguration();
+
+  config->set_workspace(workspace_new);
+  config->set_array_name("t0_1_2");
+  config->set_callset_mapping_file(workspace_new+"/callset.json");
+  config->set_vid_mapping_file(workspace_new+"/vidmap.json");
+
+  // query_row_ranges
+  RowRangeList* row_ranges = config->add_query_row_ranges();
+  RowRange* row_range = row_ranges->add_range_list();
+  row_range->set_low(0);
+  row_range->set_high(3);
+
+  // query contig_interval
+  ContigInterval* contig_interval = config->add_query_contig_intervals();
+  contig_interval->set_contig("1");
+  contig_interval->set_begin(1);
+  contig_interval->set_end(249250621);
+
+  // query_attributes
+  config->add_attributes()->assign("GT");
+  config->add_attributes()->assign("DP");
+  config->add_attributes()->assign("PID");
+
+  config->set_bypass_intersecting_intervals_phase(true);
+
+  config->set_segment_size(128);
+
+  std::string config_string;
+  CHECK(config->SerializeToString(&config_string));
+  GenomicsDB *gdb = new GenomicsDB(config_string, GenomicsDB::PROTOBUF_BINARY_STRING);
+
+  ArrowVariantCallProcessor arrow_processor;
+  gdb->query_variant_calls(arrow_processor, "", GenomicsDB::NONE);
+  void *arrow_schema = arrow_processor.arrow_schema();
+  CHECK(arrow_schema != NULL);
+  ArrowSchema *schema = reinterpret_cast<ArrowSchema *>(arrow_schema);
+  ArrowArrayView main_array_view;
+  ArrowArrayViewInitFromSchema(&main_array_view, schema, nullptr);
+
+  std::string out;
+
+  std::vector<std::vector<std::string>> results;
+  std::vector<std::vector<std::string>> expected_results = {
+    // SAMPLE_NAME
+    {"HG00141", "HG01958", "HG00141", "HG01958", "HG01530"},
+    // CHR
+    {"1", "1", "1", "1", "1"},
+    // POS
+    {"12141", "12145", "17385", "17385", "17385"},
+    // DP
+    {"NULL", "NULL", "NULL", "120", "76"},
+    // GT
+    {"C/C", "C/C", "G/A", "T/T", "G/A"},
+    // PID
+    {"NULL", "NULL", "17385_G_A", "17385_G_T", "NULL" },
+    // REF
+    {"C", "C", "G", "G", "G"},
+  };
+
+  void *arrow_array = NULL;
+  while ((arrow_array = arrow_processor.arrow_array()) != NULL) {
+    ArrowArray *array = reinterpret_cast<ArrowArray *>(arrow_array);
+    CHECK(array->n_children == schema->n_children);
+    std::vector<std::string> result;
+    for (auto i=0l; i<schema->n_children; i++) {
+      results.push_back(get_array_value(array, main_array_view.children[i]->storage_type, i));
+    }
+    ArrowVariantCallProcessor::cleanup_array(arrow_array);
+  }
+  CHECK(results == expected_results);
+  
+  // Threaded(asynchronous version) - batched by GenomicsDB column(flattened chr+pos)
+  ArrowVariantCallProcessor threaded_arrow_processor(true);
+  auto query_thread = std::async(std::launch::async, [&] {
+    gdb->query_variant_calls(threaded_arrow_processor, "", GenomicsDB::NONE);
+  });
+
+  std::vector<size_t> batch_sizes;
+  std::vector<size_t> expected_batch_sizes = {1, 1, 3};
+  std::vector<std::vector<std::vector<std::string>>> results_by_batch;
+  std::vector<std::vector<std::vector<std::string>>> expected_results_by_batch = {
+    {{"HG00141"}, {"1"}, {"12141"}, {"NULL"}, {"C/C"}, {"NULL"}, {"C"}},
+    {{"HG01958"}, {"1"}, {"12145"}, {"NULL"}, {"C/C"}, {"NULL"}, {"C"}},
+    {{"HG00141", "HG01958", "HG01530"}, {"1", "1", "1"}, {"17385", "17385", "17385"},
+     {"NULL", "120", "76"}, {"G/A", "T/T", "G/A"}, {"17385_G_A", "17385_G_T", "NULL"}, {"G", "G", "G"}}
+  };
+
+  while ((arrow_array = threaded_arrow_processor.arrow_array()) != NULL) {
+    results.clear();
+    ArrowArray *array = reinterpret_cast<ArrowArray *>(arrow_array);
+    CHECK(array->n_children == schema->n_children);
+    for (auto i=0l; i<schema->n_children; i++) {
+      results.push_back(get_array_value(array, main_array_view.children[i]->storage_type, i));
+    }
+    ArrowVariantCallProcessor::cleanup_array(arrow_array);
+    results_by_batch.push_back(results);
+  }
+  CHECK(results_by_batch == expected_results_by_batch);
+
+  query_thread.get();
+
+  // Check helper utils
+  ArrowSchema* allocated_arrow_schema;
+  CHECK(ArrowVariantCallProcessor::allocate_schema((void **)&allocated_arrow_schema, NULL) == GENOMICSDB_OK);
+  CHECK(ArrowVariantCallProcessor::allocate_schema((void **)&allocated_arrow_schema, arrow_schema) == GENOMICSDB_OK);
+
+  arrow_processor.cleanup_schema(arrow_schema);
+  delete gdb;
+}
+#endif
 
 TEST_CASE("Test genomicsdb demo test case", "[genomicsdb_demo]") {
   using namespace genomicsdb_pb;
